@@ -111,10 +111,12 @@ function loadCalendarManager() {
 
         // Wrapping this in a try/catch block, as if any of the migration code
         // fails, the app may not load.
-        try {
-            gDataMigrator.checkAndMigrate();
-        } catch (e) {
-            Components.utils.reportError("Migrator error: " + e);
+        if (getPrefSafe("calendar.migrator.enabled", true)) {
+            try {
+                gDataMigrator.checkAndMigrate();
+            } catch (e) {
+                Components.utils.reportError("Migrator error: " + e);
+            }
         }
 
         calendars = [homeCalendar];
@@ -128,7 +130,7 @@ function loadCalendarManager() {
     tree.view = calendarListTreeView;
 
     calMgr.addObserver(calendarManagerObserver);
-    composite.addObserver(calendarManagerObserver);
+    composite.addObserver(calendarManagerCompositeObserver);
     branch.addObserver("calendar.", calendarManagerObserver, false);
 
     // The calendar manager will not notify for existing calendars. Go through
@@ -139,6 +141,7 @@ function loadCalendarManager() {
 }
 
 function unloadCalendarManager() {
+    calendarManagerObserver.unload();
     var calMgr = getCalendarManager();
     var composite = getCompositeCalendar();
     var prefService = Components.classes["@mozilla.org/preferences-service;1"]
@@ -146,7 +149,7 @@ function unloadCalendarManager() {
     var branch = prefService.getBranch("").QueryInterface(Components.interfaces.nsIPrefBranch2);
 
     branch.removeObserver("calendar.", calendarManagerObserver);
-    composite.removeObserver(calendarManagerObserver);
+    composite.removeObserver(calendarManagerCompositeObserver);
     calMgr.removeObserver(calendarManagerObserver);
 }
 
@@ -166,16 +169,51 @@ function calendarListInitCategoryColors() {
     var categoryPrefBranch = prefService.getBranch("calendar.category.color.");
     var categories = categoryPrefBranch.getChildList("", {});
 
+    // check category preference name syntax
+    categories = calendarConvertObsoleteColorPrefs(categoryPrefBranch, categories);
+
     // Update all categories
     for each (var category in categories) {
         updateStyleSheetForObject(category, gCachedStyleSheet);
     }
 }
 
+/**
+ * Remove illegally formatted category names from the array coloredCategories
+ * so they don't cause CSS errors.  For each illegal colored category c, if
+ * its color preference has not yet been replaced with a converted preference
+ * with key formatStringForCSSRule(c), create the preference with the
+ * converted key and with the previous preference value, and clear the old
+ * preference.  (For most users who upgrade and do not later add colors with a
+ * downgrade version, this should convert any illegal preferences once, so
+ * future runs have no illegal preferences.)
+ * @param categoryPrefBranch prefBranch for "calendar.category.color."
+ * @param coloredCategories array of preference name suffixes under the prefBranch.
+ * @return same array with each illegal name replaced with formatted name if
+ * it doesn't already exist, or simply removed from array if it does.
+ */
+function calendarConvertObsoleteColorPrefs(categoryPrefBranch, coloredCategories) {
+    for (var i in coloredCategories) {
+        var category = coloredCategories[i];
+        if (category.search(/[^_0-9a-z-]/) != -1) {
+            var categoryFix = formatStringForCSSRule(category);
+            if (!categoryPrefBranch.prefHasUserValue(categoryFix)) {
+                var color = categoryPrefBranch.getCharPref(category);
+                categoryPrefBranch.setCharPref(categoryFix, color);
+                categoryPrefBranch.clearUserPref(category); // not usable
+                coloredCategories[i] = categoryFix;  // replace illegal name
+            } else {
+                coloredCategories.splice(i, 1); // remove illegal name
+            }
+        }
+    }
+    return coloredCategories;
+}
+
 function calendarListUpdateColor(aCalendar) {
     var selectorPrefix = "treechildren::-moz-tree-cell";
 
-    var color = getCalendarManager().getCalendarPref(aCalendar, "color");
+    var color = aCalendar.getProperty("color");
     if (!color) {
         return;
     }
@@ -206,6 +244,12 @@ var calendarListTreeView = {
     treebox: null,
     mContextElement: null,
 
+    QueryInterface: function cLTV_QueryInterface(aIID) {
+        return doQueryInterface(this, calendarListTreeView.__proto__, aIID,
+                                [Components.interfaces.nsISupports,
+                                 Components.interfaces.nsITreeView]);
+    },
+
     /**
      * High-level calendar tree manipulation
      */
@@ -219,6 +263,15 @@ var calendarListTreeView = {
         return -1;
     },
 
+    findIndexByUri: function cLTV_findIndexByUri(aUri) {
+        for (var i = 0; i < this.mCalendarList.length; i++) {
+            if (this.mCalendarList[i].uri.equals(aUri)) {
+                return i;
+            }
+        }
+        return -1;
+    },
+
     addCalendar: function cLTV_addCalendar(aCalendar) {
         var composite = getCompositeCalendar();
         this.mCalendarList.push(aCalendar);
@@ -282,6 +335,8 @@ var calendarListTreeView = {
         return this.mCalendarList.length;
     },
 
+    getRowProperties: function cLTV_getRowProperties(aRow, aProps) {},
+
     getCellProperties: function cLTV_getCellProperties(aRow, aCol, aProps) {
         var calendar = this.mCalendarList[aRow];
         var composite = getCompositeCalendar();
@@ -295,26 +350,69 @@ var calendarListTreeView = {
                 }
                 break;
             case "calendar-list-tree-color":
-                var color = getCalendarManager().getCalendarPref(calendar,
-                                                                 "color");
-                color = "color-" + (color ? color.substr(1) : "default");
-                aProps.AppendElement(getAtomFromService(color));
+                // Get the calendar color
+                var color = calendar.getProperty("color");
+                color = color && color.substr(1);
+
+                // Set up the calendar color (background)
+                var bgColorProp = "color-" + (color || "default");
+                aProps.AppendElement(getAtomFromService(bgColorProp));
+
+                // Set a property to get the contrasting text color (foreground)
+                var fgColorProp = getContrastingTextColor(color || "a8c2e1");
+                aProps.AppendElement(getAtomFromService(fgColorProp));
+
+                // Set up the readonly symbol
+                if (calendar.readOnly) {
+                    aProps.AppendElement(getAtomFromService("readOnly"));
+                }
+
                 break;
-       }
-    },
-
-    cycleCell: function cLTV_cycleCell(aRow, aCol) {
-        var calendar = this.mCalendarList[aRow];
-        var composite = getCompositeCalendar();
-
-        if (composite.getCalendar(calendar.uri)) {
-            composite.removeCalendar(calendar.uri);
-        } else {
-            composite.addCalendar(calendar);
         }
-        this.treebox.invalidateRow(aRow);
     },
 
+    getColumnProperties: function cLTV_getColumnProperties(a, aProps) {},
+
+    isContainer: function cLTV_isContainer(aRow) {
+        return false;
+    },
+
+    isContainerOpen: function cLTV_isContainerOpen(aRow) {
+        return false;
+    },
+
+    isContainerEmpty: function cLTV_isContainerEmpty(aRow) {
+        return false;
+    },
+
+    isSeparator: function cLTV_isSeparator(aRow) {
+        return false;
+    },
+
+    isSorted: function cLTV_isSorted(aRow) {
+        return false;
+    },
+
+    canDrop: function cLTV_canDrop(aRow, aOrientation) {
+        return false;
+    },
+
+    drop: function cLTV_drop(aRow, aOrientation) {},
+
+    getParentIndex: function cLTV_getParentIndex(aRow) {
+        return -1;
+    },
+
+    hasNextSibling: function cLTV_hasNextSibling(aRow, aAfterIndex) {},
+
+    getLevel: function cLTV_getLevel(aRow) {
+        return 0;
+    },
+
+    getImageSrc: function cLTV_getImageSrc(aRow, aOrientation) {},
+
+    getProgressMode: function cLTV_getProgressMode(aRow, aCol) {},
+
     getCellValue: function cLTV_getCellValue(aRow, aCol) {
         var calendar = this.mCalendarList[aRow];
         var composite = getCompositeCalendar();
@@ -322,22 +420,9 @@ var calendarListTreeView = {
         switch (aCol.id) {
             case "calendar-list-tree-checkbox":
                 return composite.getCalendar(calendar.uri) ? "true" : "false";
-        }
-        return null;
-    },
-
-    setCellValue: function cLTV_setCellValue(aRow, aCol, aValue) {
-        var calendar = this.mCalendarList[aRow];
-        var composite = getCompositeCalendar();
-
-        switch (aCol.id) {
-            case "calendar-list-tree-checkbox":
-                if (aValue == "true") {
-                    composite.addCalendar(calendar);
-                } else {
-                    composite.removeCalendar(calendar);
-                }
-                return aValue;
+            case "calendar-list-tree-color":
+                // The value of this cell shows the calendar readonly state
+                return (calendar.readOnly ? "true" : "false");
         }
         return null;
     },
@@ -354,39 +439,66 @@ var calendarListTreeView = {
         return "";
     },
 
-    getImageSrc: function cLTV_getImageSrc(aRow, aCol) {
-        return null;
+    setTree: function cLTV_setTree(aTreeBox) {
+        this.treebox = aTreeBox;
+    },
+
+    toggleOpenState: function cLTV_toggleOpenState(aRow) {},
+
+    cycleHeader: function cLTV_cycleHeader(aCol) { },
+
+    cycleCell: function cLTV_cycleCell(aRow, aCol) {
+        var calendar = this.mCalendarList[aRow];
+        var composite = getCompositeCalendar();
+
+        switch (aCol.id) {
+            case "calendar-list-tree-checkbox":
+                if (composite.getCalendar(calendar.uri)) {
+                    composite.removeCalendar(calendar.uri);
+                } else {
+                    composite.addCalendar(calendar);
+                }
+                break;
+            case "calendar-list-tree-color":
+                // Clicking on the color should toggle the readonly state.
+                calendar.readOnly = !calendar.readOnly;
+                break;
+        }
+        this.treebox.invalidateRow(aRow);
     },
 
     isEditable: function cLTV_isEditable(aRow, aCol) {
         return false;
     },
 
-    setTree: function cLTV_setTree(aTreeBox) {
-        this.treebox = aTreeBox;
+    setCellValue: function cLTV_setCellValue(aRow, aCol, aValue) {
+        var calendar = this.mCalendarList[aRow];
+        var composite = getCompositeCalendar();
+
+        switch (aCol.id) {
+            case "calendar-list-tree-checkbox":
+                if (aValue == "true") {
+                    composite.addCalendar(calendar);
+                } else {
+                    composite.removeCalendar(calendar);
+                }
+                break;
+            case "calendar-list-tree-color":
+                calendar.readOnly = (aValue == "true");
+                break;
+            default:
+                return null;
+        }
+        return aValue;
     },
 
-    isContainer: function cLTV_isContainer(aRow) {
-        return false;
-    },
+    setCellText: function cLTV_setCellText(aRow, aCol, aValue) {},
 
-    isSeparator: function cLTV_isSeparator(aRow) {
-        return false;
-    },
+    performAction: function cLTV_performAction(aAction) {},
 
-    isSorted: function cLTV_isSorted(aRow) {
-        return false;
-    },
+    performActionOnRow: function cLTV_performActionOnRow(aAction, aRow) {},
 
-    getLevel: function cLTV_getLevel(aRow) {
-        return 0;
-    },
-
-    getRowProperties: function cLTV_getRowProperties(aRow, aProps) {},
-
-    getColumnProperties: function cLTV_getColumnProperties(aCol, aProps) {},
-
-    cycleHeader: function cLTV_cycleHeader(aCol) { },
+    performActionOnCell: function cLTV_performActionOnCell(aAction, aRow, aCol) {},
 
     /**
      * Calendar Tree Events
@@ -401,7 +513,7 @@ var calendarListTreeView = {
                 if (this.tree.currentIndex > -1 ) {
                     var cbCol =
                         this.treebox.columns
-                            .getNamedColumn("list-calendarsr-tree-checkbox");
+                            .getNamedColumn("calendar-list-tree-checkbox");
                     this.cycleCell(this.tree.currentIndex, cbCol);
                 }
                 break;
@@ -464,6 +576,10 @@ var calendarListTreeView = {
             column: col && col.value
         };
 
+        // Only enable calendar search if there's actually the chance of finding something:
+        document.getElementById("list-calendars-context-find").setAttribute(
+            "collapsed", (getCalendarSearchService().getProviders({}).length > 0 ? "false" : "true"));
+
         if (calendar) {
             document.getElementById("list-calendars-context-edit")
                     .removeAttribute("disabled");
@@ -487,110 +603,16 @@ var calendarListTreeView = {
     }
 };
 
-var calendarManagerObserver = {
-    mDefaultCalendarItem: null,
-
-    QueryInterface: function cMO_QueryInterface(aIID) {
-        if (!aIID.equals(Components.interfaces.calICalendarManagerObserver) &&
-            !aIID.equals(Components.interfaces.calICompositeObserver) &&
+var calendarManagerCompositeObserver = {
+    QueryInterface: function cMCO_QueryInterface(aIID) {
+        if (!aIID.equals(Components.interfaces.calICompositeObserver) &&
             !aIID.equals(Components.interfaces.calIObserver) &&
-            !aIID.equals(Components.interfaces.nsIObserver) &&
             !aIID.equals(Components.interfaces.nsISupports)) {
             throw Components.results.NS_ERROR_NO_INTERFACE;
         }
         return this;
     },
 
-    /**
-     * Set up the UI for a new calendar.
-     *
-     * @param aCalendar     The calendar to add.
-     */
-    initializeCalendar: function cMO_initializeCalendar(aCalendar) {
-        var calendars = getCalendarManager().getCalendars({});
-        calendarListTreeView.addCalendar(aCalendar);
-
-        updateStyleSheetForObject(aCalendar, gCachedStyleSheet);
-        calendarListUpdateColor(aCalendar);
-
-        // Watch the calendar for changes, to ensure its visibility when adding
-        // or changing items.
-        aCalendar.addObserver(this);
-
-        // Make sure we can delete calendars when there is more than one.
-        if (calendars.length > 1) {
-            document.getElementById("calendar_delete_calendar_command")
-                    .removeAttribute("disabled");
-        }
-
-        if (aCalendar.canRefresh) {
-            document.getElementById("calendar_reload_remote_calendars")
-                    .removeAttribute("disabled");
-        }
-    },
-
-    // calICalendarManagerObserver
-    onCalendarRegistered: function cMO_onCalendarRegistered(aCalendar) {
-        this.initializeCalendar(aCalendar);
-        getCompositeCalendar().addCalendar(aCalendar);
-    },
-
-    onCalendarUnregistering: function cMO_onCalendarUnregistering(aCalendar) {
-        var calendars = getCalendarManager().getCalendars({});
-
-        calendarListTreeView.removeCalendar(aCalendar);
-        aCalendar.removeObserver(this);
-
-        // We want to make sure its not possible to delete the last calendar.
-        // Since at this point the current calendar hasn't been deleted yet,
-        // start disabling when there are two calendars.
-        if (calendars.length <= 2) {
-            document.getElementById("calendar_delete_calendar_command")
-                    .setAttribute("disabled", true);
-        }
-
-        if (aCalendar.canRefresh) {
-            // This may be the last refreshable calendar. In that case, disable
-            // the possibility to reload remote calendars.
-            function calCanRefresh(cal) {
-                return (cal.canRefresh && !cal.uri.equals(aCalendar.uri));
-            }
-            if (!calendars.some(calCanRefresh)) {
-                document.getElementById("calendar_reload_remote_calendars")
-                        .setAttribute("disabled", true);
-            }
-        }
-    },
-
-    onCalendarDeleting: function cMO_onCalendarDeleting(aCalendar) {
-        // Make sure the calendar is removed from the composite calendar
-        getCompositeCalendar().removeCalendar(aCalendar.uri);
-    },
-
-    onCalendarPrefChanged: function cMO_onCalendarPrefChanged(aCalendar,
-                                                              aName,
-                                                              aValue,
-                                                              aOldValue) {
-        switch (aName) {
-            case "color":
-                updateStyleSheetForObject(aCalendar, gCachedStyleSheet);
-                calendarListUpdateColor(aCalendar);
-                // Fall through, update item in any case
-            case "name":
-                calendarListTreeView.updateCalendar(aCalendar);
-                break;
-        }
-    },
-
-    onCalendarPrefDeleting: function cMO_onCalendarPrefDeleting(aCalendar,
-                                                                aName) {
-        // Since the old value is not used directly in onCalendarPrefChanged,
-        // but should not be the same as the value, set it to a different
-        // value.
-        this.onCalendarPrefChanged(aCalendar, aName, null, true);
-    },
-
-    // calICompositeObserver
     onCalendarAdded: function cMO_onCalendarAdded(aCalendar) {
         // Make sure the checkbox state is updated
         var index = calendarListTreeView.findIndex(aCalendar);
@@ -612,17 +634,162 @@ var calendarManagerObserver = {
     onEndBatch: function cMO_onEndBatch() { },
     onLoad: function cMO_onLoad() { },
 
+    // TODO: remove these temporary caldav exclusions when it is safe to do so
+    // needed to allow cadav refresh() to update w/o forcing visibility    
     onAddItem: function cMO_onAddItem(aItem) {
-        ensureCalendarVisible(aItem.calendar);
+        if (aItem.calendar.type != "caldav") {
+            ensureCalendarVisible(aItem.calendar);
+        }
     },
 
     onModifyItem: function cMO_onModifyItem(aNewItem, aOldItem) {
-        ensureCalendarVisible(aNewItem.calendar);
+        if (aNewItem.calendar.type != "caldav") {
+            ensureCalendarVisible(aNewItem.calendar);
+        }
     },
 
     onDeleteItem: function cMO_onDeleteItem(aDeletedItem) { },
     onError: function cMO_onError(aErrNo, aMessage) { },
 
+    onPropertyChanged: function cMO_onPropertyChanged(aCalendar,
+                                                      aName,
+                                                      aValue,
+                                                      aOldValue) {},
+    onPropertyDeleting: function cMO_onPropertyDeleting(aCalendar,
+                                                        aName) {}
+}
+
+var calendarManagerObserver = {
+    mDefaultCalendarItem: null,
+
+    QueryInterface: function cMO_QueryInterface(aIID) {
+        if (!aIID.equals(Components.interfaces.calICalendarManagerObserver) &&
+            !aIID.equals(Components.interfaces.calIObserver) &&
+            !aIID.equals(Components.interfaces.nsIObserver) &&
+            !aIID.equals(Components.interfaces.nsISupports)) {
+            throw Components.results.NS_ERROR_NO_INTERFACE;
+        }
+        return this;
+    },
+
+    /**
+     * Set up the UI for a new calendar.
+     *
+     * @param aCalendar     The calendar to add.
+     */
+    initializeCalendar: function cMO_initializeCalendar(aCalendar) {
+        calendarListTreeView.addCalendar(aCalendar);
+
+        updateStyleSheetForObject(aCalendar, gCachedStyleSheet);
+        calendarListUpdateColor(aCalendar);
+
+        // Watch the calendar for changes, to ensure its visibility when adding
+        // or changing items.
+        aCalendar.addObserver(this);
+
+        // Update the calendar commands for number of remote calendars and for
+        // more than one calendar
+        document.commandDispatcher.updateCommands("calendar_commands");
+    },
+    
+    unload: function cMO_unload() {
+        var calendars = getCalendarManager().getCalendars({});
+        for each (var calendar in calendars) {
+            calendar.removeObserver(this);
+        }
+    },
+
+    setupWritableCalendars: function cMO_setupWritableCalendars() {
+        var nodes = document.getElementsByAttribute("disable-when-no-writable-calendars", "true");
+        for (var i = 0; i < nodes.length; i++) {
+            if (this.mWritableCalendars < 1) {
+                nodes[i].setAttribute("disabled", "true");
+            } else {
+                nodes[i].removeAttribute("disabled");
+            }
+        }
+    },
+
+    // calICalendarManagerObserver
+    onCalendarRegistered: function cMO_onCalendarRegistered(aCalendar) {
+        this.initializeCalendar(aCalendar);
+        var composite = getCompositeCalendar();
+        var inComposite = aCalendar.getProperty(composite.prefPrefix +
+                                                "-in-composite");
+        if ((inComposite === null) || inComposite) {
+            composite.addCalendar(aCalendar);
+        }
+    },
+
+    onCalendarUnregistering: function cMO_onCalendarUnregistering(aCalendar) {
+        var calendars = getCalendarManager().getCalendars({});
+
+        calendarListTreeView.removeCalendar(aCalendar);
+        aCalendar.removeObserver(this);
+
+        // Make sure the calendar is removed from the composite calendar
+        getCompositeCalendar().removeCalendar(aCalendar.uri);
+
+        // Update commands to disallow deleting the last calendar and only
+        // allowing reload remote calendars when there are remote calendars.
+        document.commandDispatcher.updateCommands("calendar_commands");
+    },
+
+    onCalendarDeleting: function cMO_onCalendarDeleting(aCalendar) {
+    },
+
+    // calIObserver. Note that each registered calendar uses this observer, not
+    // only the composite calendar.
+    onStartBatch: function cMO_onStartBatch() { },
+    onEndBatch: function cMO_onEndBatch() { },
+    onLoad: function cMO_onLoad() { },
+
+    // TODO: remove these temporary caldav exclusions when it is safe to do so
+    // needed to allow cadav refresh() to update w/o forcing visibility    
+    onAddItem: function cMO_onAddItem(aItem) {
+        if (aItem.calendar.type != "caldav") {
+            ensureCalendarVisible(aItem.calendar);
+        }
+    },
+
+    onModifyItem: function cMO_onModifyItem(aNewItem, aOldItem) {
+        if (aNewItem.calendar.type != "caldav") {
+            ensureCalendarVisible(aNewItem.calendar);
+        }
+    },
+
+    onDeleteItem: function cMO_onDeleteItem(aDeletedItem) { },
+    onError: function cMO_onError(aErrNo, aMessage) { },
+
+    onPropertyChanged: function cMO_onPropertyChanged(aCalendar,
+                                                      aName,
+                                                      aValue,
+                                                      aOldValue) {
+        switch (aName) {
+            case "color":
+                updateStyleSheetForObject(aCalendar, gCachedStyleSheet);
+                calendarListUpdateColor(aCalendar);
+                // Fall through, update item in any case
+            case "name":
+                calendarListTreeView.updateCalendar(aCalendar);
+                break;
+            case "readOnly":
+                calendarListTreeView.updateCalendar(aCalendar);
+                // Fall through, update commands in any cases.
+            case "requiresNetwork":
+                document.commandDispatcher.updateCommands("calendar_commands");
+                break;
+        }
+    },
+
+    onPropertyDeleting: function cMO_onPropertyDeleting(aCalendar,
+                                                        aName) {
+        // Since the old value is not used directly in onPropertyChanged,
+        // but should not be the same as the value, set it to a different
+        // value.
+        this.onPropertyChanged(aCalendar, aName, null, null);
+    },
+
     // nsIObserver
     observe: function cMO_observe(aSubject, aTopic, aPrefName) {
 
@@ -639,7 +806,6 @@ var calendarManagerObserver = {
                     view.goToDay(day);
                 }
 
-
                 if (isSunbird()) {
                     refreshEventTree();
                 }
@@ -674,3 +840,64 @@ var calendarManagerObserver = {
         }
     }
 };
+
+function openCalendarSubscriptionsDialog() {
+    // the dialog will reset this to auto when it is done loading
+    window.setCursor("wait");
+ 
+    // open the dialog modally
+    window.openDialog("chrome://calendar/content/calendar-subscriptions-dialog.xul",
+                      "_blank",
+                      "chrome,titlebar,modal,resizable");
+}
+
+/**
+ * Calendar Offline Manager
+ */
+var calendarOfflineManager = {
+    QueryInterface: function cOM_QueryInterface(aIID) {
+        return doQueryInterface(this, calendarOfflineManager.prototype, aIID,
+                                [Components.interfaces.nsIObserver, Components.interfaces.nsISupports]);
+    },
+
+    init: function cOM_init() {
+        if (this.initialized) {
+            throw Components.results.NS_ERROR_ALREADY_INITIALIZED;
+        }
+        var os = Components.classes["@mozilla.org/observer-service;1"]
+                           .getService(Components.interfaces.nsIObserverService);
+        os.addObserver(this, "network:offline-status-changed", false);
+
+        this.updateOfflineUI(!this.isOnline());
+        this.initialized = true;
+    },
+
+    uninit: function cOM_uninit() {
+        if (!this.initialized) {
+            throw Components.results.NS_ERROR_NOT_INITIALIZED;
+        }
+        var os = Components.classes["@mozilla.org/observer-service;1"]
+                           .getService(Components.interfaces.nsIObserverService);
+        os.removeObserver(this, "network:offline-status-changed", false);
+        this.initialized = false;
+    },
+
+    isOnline: function cOM_isOnline() {
+        return (!getIOService().offline);
+
+    },
+
+    updateOfflineUI: function cOM_updateOfflineUI(aIsOffline) {
+        // Refresh the current view
+        currentView().goToDay(currentView().selectedDay);
+
+        // Set up disabled locks for offline
+        document.commandDispatcher.updateCommands("calendar_commands");
+    },
+
+    observe: function cOM_observe(aSubject, aTopic, aState) {
+        if (aTopic == "network:offline-status-changed") {
+            this.updateOfflineUI(aState == "offline");
+        }
+    }
+};
