@@ -164,6 +164,13 @@ var calendarViewController = {
                     }
                 }
             }
+
+            // If the item contains attendees then they need to be notified
+            if (instance.hasProperty("X-MOZ-SEND-INVITATIONS") &&
+               (instance.getProperty("X-MOZ-SEND-INVITATIONS") == "TRUE")) {
+               sendItipInvitation(instance);
+            }
+
             doTransaction('modify', instance, instance.calendar, aOccurrence, null);
         } else {
             // prompt for choice between occurrence and master for recurrent items
@@ -241,23 +248,76 @@ var calendarViewController = {
     }
 };
 
+/**
+ * This function provides a neutral way to switch between views.
+ * XXX Kind of confusing. This function calls the app specific function, which
+ * again calls the common switchToView function. They should be consolidated in
+ * a different bug.
+ */
+function showCalendarView(type) {
+    if (isSunbird()) {
+        gCalendarWindow.switchToView(type);
+    } else {
+        ltnShowCalendarView(type);
+    }
+}
+
+/**
+ * This function acts like the above, but does not bring the view to the front
+ * if the application is showing other elements (i.e Lightning).
+ */
+function selectCalendarView(type) {
+    if (isSunbird()) {
+        gCalendarWindow.switchToView(type);
+    } else {
+        ltnSelectCalendarView(type);
+    }
+}
+
+/**
+ * This function does the common steps to switch between views. Should be called
+ * from app-specific view switching functions
+ */
 function switchToView(aViewType) {
     var viewDeck = getViewDeck();
     var selectedDay;
     var currentSelection = [];
+
+    // Set up the view commands
+    var views = viewDeck.childNodes;
+    for (var i = 0; i < views.length; i++) {
+        var view = views[i];
+        var commandId = "calendar_" + view.id + "_command";
+        var command = document.getElementById(commandId);
+        if (view.id == aViewType + "-view") {
+            command.setAttribute("checked", "true");
+        } else {
+            command.removeAttribute("checked");
+        }
+    }
+
+    // Disable the menuitem when not in day or week view.
+    var rotated = document.getElementById("calendar_toggle_orientation_command");
+    if (aViewType == "day" || aViewType == "week") {
+        rotated.removeAttribute("disabled");
+    } else {
+        rotated.setAttribute("disabled", "true");
+    }
+
     try {
         selectedDay = viewDeck.selectedPanel.selectedDay;
         currentSelection = viewDeck.selectedPanel.getSelectedItems({});
-    } catch(ex) {
+    } catch (ex) {
         // This dies if no view has even been chosen this session, but that's
         // ok because we'll just use now() below.
-    } 
+    }
 
-    if (!selectedDay)
+    if (!selectedDay) {
         selectedDay = now();
+    }
 
     // Anyone wanting to plug in a view needs to follow this naming scheme
-    var view = document.getElementById(aViewType+"-view");
+    var view = document.getElementById(aViewType + "-view");
     viewDeck.selectedPanel = view;
 
     var compositeCal = getCompositeCalendar();
@@ -299,7 +359,7 @@ function getSelectedDay() {
 
 var gMidnightTimer;
 
-/** Creates a timer that will fire after midnight.  Pass in a function as 
+/** Creates a timer that will fire after midnight.  Pass in a function as
  * aRefreshCallback that should be called at that time.
  */
 function scheduleMidnightUpdate(aRefreshCallback) {
@@ -357,17 +417,22 @@ function getStyleSheet(aStyleSheetPath) {
     return null;
 }
 
-// Updates the style rules for a particular object.  If the object is a
-// category (and hence doesn't have a uri), we set the border color.  If
-// it's a calendar, we set the background color
+/**
+ * Updates the style rules for a particular object.  If the object is a
+ * category (and hence doesn't have a uri), we set the category bar color.
+ * If it's a calendar, we set the background color and contrasting text color.
+ * @param aObject either a calendar (with a .uri), or the category color
+ * pref key suffix [the non-unicode part after "calendar.category.color.",
+ * equivalent to formatStringForCSSRule(categoryNameInUnicode)].
+ */
 function updateStyleSheetForObject(aObject, aSheet) {
     var selectorPrefix, name, ruleUpdaterFunc;
     if (aObject.uri) {
-        // This is a calendar, so we're going to set the background color
+        // For a calendar, set background and contrasting text colors
         name = aObject.uri.spec;
         selectorPrefix = "item-calendar=";
         ruleUpdaterFunc = function calendarRuleFunc(aRule, aIndex) {
-            var color = getCalendarManager().getCalendarPref(aObject, 'color');
+            var color = aObject.getProperty('color');
             if (!color) {
                 color = "#A8C2E1";
             }
@@ -375,14 +440,14 @@ function updateStyleSheetForObject(aObject, aSheet) {
             aRule.style.color = getContrastingTextColor(color);
         };
     } else {
-        // This is a category, where we set the border.  Also note that we 
-        // use the ~= selector, since there could be multiple categories
-        name = aObject.replace(' ','_');
+        // For a category, set the category bar color.  Also note that
+        // it uses the ~= selector, since there could be multiple categories.
+        name = aObject;
         selectorPrefix = "item-category~=";
         ruleUpdaterFunc = function categoryRuleFunc(aRule, aIndex) {
-            var color = getPrefSafe("calendar.category.color."+aObject, null);
+            var color = getPrefSafe("calendar.category.color."+name, null);
             if (color) {
-                aRule.style.border = color + " solid 2px";
+                aRule.style.backgroundColor = color;
             } else {
                 aSheet.deleteRule(aIndex);
             }
@@ -457,3 +522,157 @@ function getMinimonth() {
     var sbMinimonth = document.getElementById("lefthandcalendar");
     return sbMinimonth || document.getElementById("ltnMinimonth");
 }
+
+/**
+ * Update the view orientation based on the checked state of the command
+ */
+function toggleOrientation() {
+    var cmd = document.getElementById("calendar_toggle_orientation_command");
+    var newValue = (cmd.getAttribute("checked") == "true" ? "false" : "true");
+    cmd.setAttribute("checked", newValue);
+
+    var deck = getViewDeck();
+    for each (var view in deck.childNodes) {
+        view.rotated = (newValue == "true");
+    }
+
+    // orientation refreshes automatically
+}
+
+/**
+ * Toggle the workdays only checkbox and refresh the current view
+ */
+function toggleWorkdaysOnly() {
+    var cmd = document.getElementById("calendar_toggle_workdays_only_command");
+    var newValue = (cmd.getAttribute("checked") == "true" ? "false" : "true");
+    cmd.setAttribute("checked", newValue);
+
+    var deck = getViewDeck();
+    for each (var view in deck.childNodes) {
+        view.workdaysOnly = (newValue == "true");
+    }
+
+    // Refresh the current view
+    currentView().goToDay(currentView().selectedDay);
+}
+
+/**
+ * Toggle the tasks in view checkbox and refresh the current view
+ */
+function toggleTasksInView() {
+    var cmd = document.getElementById("calendar_toggle_tasks_in_view_command");
+    var newValue = (cmd.getAttribute("checked") == "true" ? "false" : "true");
+    cmd.setAttribute("checked", newValue);
+
+    var deck = getViewDeck();
+    for each (var view in deck.childNodes) {
+        view.tasksInView = (newValue == "true");
+    }
+
+    // Refresh the current view
+    currentView().goToDay(currentView().selectedDay);
+}
+
+/**
+ * Provides a neutral way to go to the current day
+ */
+function goToDate(aDate) {
+    getMinimonth().value = aDate.jsDate;
+    currentView().goToDay(aDate);
+}
+
+/**
+ * Sets up menuitems for the views
+ */
+function initializeViews() {
+    // Set up the checkbox variables. Do not use the typical toggle*() functions
+    // because those will actually toggle the current value.
+    const kWorkdaysCommand = "calendar_toggle_workdays_only_command";
+    const kTasksInViewCommand = "calendar_toggle_tasks_in_view_command";
+    const kOrientation = "calendar_toggle_orientation_command";
+    const kHideCompleted = "hide-completed-checkbox";
+    var workdaysOnly = (document.getElementById(kWorkdaysCommand)
+                                .getAttribute("checked") == "true");
+
+    var tasksInView = (document.getElementById(kTasksInViewCommand)
+                               .getAttribute("checked") == "true");
+    var rotated = (document.getElementById(kOrientation)
+                           .getAttribute("checked") == "true");
+    var showCompleted = !document.getElementById(kHideCompleted).checked;
+
+    var deck = getViewDeck();
+    for each (var view in deck.childNodes) {
+        view.workdaysOnly = workdaysOnly;
+        view.tasksInView = tasksInView;
+        view.showCompleted = showCompleted;
+        view.rotated = rotated;
+    }
+
+    // Restore the last shown view
+    var deck = getViewDeck();
+    if (deck.selectedIndex < 0) {
+        // No deck item was selected beforehand, default to week view.
+        selectCalendarView("week");
+    } else {
+        var viewNode = deck.childNodes[deck.selectedIndex];
+        var viewName = viewNode.id.replace(/-view/, "");
+        selectCalendarView(viewName);
+    }
+}
+
+/**
+ *  Deletes items currently selected in the view.
+ */
+function deleteSelectedEvents() {
+    var selectedItems = currentView().getSelectedItems({});
+    calendarViewController.deleteOccurrences(selectedItems.length,
+                                             selectedItems,
+                                             false,
+                                             false);
+}
+
+/**
+ *  Edit the items currently selected in the view.
+ */
+function editSelectedEvents() {
+    var selectedItems = currentView().getSelectedItems({});
+    if (selectedItems && selectedItems.length >= 1) {
+        modifyEventWithDialog(getOccurrenceOrParent(selectedItems[0]));
+    }
+}
+
+/**
+ * Select all events from all calendars
+ */
+function selectAllEvents() {
+    var items = [];
+    var listener = {
+        onOperationComplete: function selectAll_ooc(aCalendar, aStatus, 
+                                                    aOperationType, aId, 
+                                                    aDetail) {
+            currentView().setSelectedItems(items.length, items, false);
+        },
+        onGetResult: function selectAll_ogr(aCalendar, aStatus, aItemType, 
+                                            aDetail, aCount, aItems) {
+            for each (var item in aItems) {
+                items.push(item);
+            }
+        }
+    };
+
+    var composite = getCompositeCalendar();
+    var filter = composite.ITEM_FILTER_COMPLETED_ALL |
+                 composite.ITEM_FILTER_CLASS_OCCURRENCES;
+
+    if (currentView().tasksInView) {
+        filter |= composite.ITEM_FILTER_TYPE_ALL; 
+    } else {
+        filter |= composite.ITEM_FILTER_TYPE_EVENT;
+    }
+
+    // Need to move one day out to get all events
+    var end = currentView().endDay.clone();
+    end.day += 1;
+
+    composite.getItems(filter, 0, currentView().startDay, end, listener);
+}
