@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Daniel Boelzle <daniel.boelzle@sun.com>
+ *   Philipp Kewisch <mozilla@kewis.ch>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -60,7 +61,10 @@ function calWcapCalendar_encodeAttendee(att)
     var params = encodeAttr(att.rsvp ? "TRUE" : "FALSE", "RSVP", "");
     params = encodeAttr(att.participationStatus, "PARTSTAT", params);
     params = encodeAttr(att.role, "ROLE", params);
-    params = encodeAttr(att.commonName, "CN", params);
+    var cn = att.commonName;
+    if (cn) {
+        params = encodeAttr(cn.replace(/[;:]/g, " "), "CN", params);
+    }
     return encodeAttr(att.id, null, params);
 };
 
@@ -68,10 +72,14 @@ calWcapCalendar.prototype.encodeNscpTzid =
 function calWcapCalendar_encodeNscpTzid(dateTime)
 {
     var params = "X-NSCP-ORIGINAL-OPERATION=X-NSCP-WCAP-PROPERTY-";
-    if (!dateTime || !dateTime.isValid || dateTime.isDate || (dateTime.timezone == "floating")) {
+    if (!dateTime || !dateTime.isValid || dateTime.timezone.isUTC) {
         params += "DELETE^";
+    } else if (dateTime.timezone.isFloating) {
+        // cs cannot cope with floating, always enforces server tz,
+        // better use user's tz then:
+        params += ("REPLACE^" + encodeURIComponent(this.defaultTimezone));
     } else {
-        params += ("REPLACE^" + encodeURIComponent(this.getAlignedTimezone(dateTime.timezone)));
+        params += ("REPLACE^" + encodeURIComponent(this.getAlignedTzid(dateTime.timezone)));
     }
     return params;
 };
@@ -90,9 +98,7 @@ function calWcapCalendar_getRecurrenceParams(
         for each (var rItem in rItems) {
             var isNeg = rItem.isNegative;
             if (rItem instanceof Components.interfaces.calIRecurrenceRule) {
-                var rule = ("\"" + encodeURIComponent(
-                                rItem.icalProperty.valueAsIcalString) +
-                            "\"");
+                var rule = ("\"" + encodeURIComponent(rItem.icalProperty.valueAsIcalString) + "\"");
                 if (isNeg)
                     out_exrules.value.push(rule);
                 else
@@ -277,7 +283,7 @@ function equalDatetimes(one, two) {
 function identicalDatetimes(one, two) {
     return ((!one && !two) ||
             (equalDatetimes(one, two) &&
-             (one.timezone == two.timezone)));
+             compareObjects(one.timezone, two.timezone)));
 }
 
 // @return null if nothing has changed else value to be written
@@ -318,7 +324,7 @@ function calWcapCalendar_storeItem(bAddItem, item, oldItem, request, netRespFunc
     var params = "";
     
     var calId = this.calId;
-    if (this.isInvitation(item)) { // REPLY
+    if (!bAddItem && this.isInvitation(item)) { // REPLY
         method = METHOD_REPLY;
         var att = getAttendeeByCalId(item.getAttendees({}), calId);
         if (att) {
@@ -488,9 +494,8 @@ function calWcapCalendar_storeItem(bAddItem, item, oldItem, request, netRespFunc
             case "IN-PROCESS":   params += "&status=5"; break;
             case "DRAFT":        params += "&status=6"; break;
             case "FINAL":        params += "&status=7"; break;
-            default:
-                params += "&status=3"; // NEEDS-ACTION
-                break;
+            default: // reset to default
+                params += (bIsEvent ? "&status=0" : "&status=3"); break;
             }
         }
         
@@ -564,11 +569,13 @@ function calWcapCalendar_storeItem(bAddItem, item, oldItem, request, netRespFunc
     else {
         var someDate = (item.startDate || item.entryDate || item.dueDate);
         if (someDate) {
-            // provide some date: eMail notification dates are influenced by this parameter...
-            params += ("&tzid=" + encodeURIComponent(
-                           this.getAlignedTimezone(someDate.timezone)));
+            someTz = someDate.timezone;
+            if (!someTz.isUTC && !someTz.isFloating) {
+                // provide some tzid: eMail notification dates are influenced by this parameter
+                params += ("&tzid=" + encodeURIComponent(this.getAlignedTzid(someTz)));
+            }
         }
-        
+
         if (item.id)
             params += ("&uid=" + encodeURIComponent(item.id));
         
@@ -579,13 +586,7 @@ function calWcapCalendar_storeItem(bAddItem, item, oldItem, request, netRespFunc
         if (bIsParent)
             params += "&mod=4"; // THIS AND ALL INSTANCES
         else {
-            var rid = item.recurrenceId;
-            if (rid.isDate) {
-                // cs does not accept DATE:
-                rid = rid.clone();
-                rid.isDate = false;
-            }
-            params += ("&mod=1&rid=" + getIcalUTC(rid)); // THIS INSTANCE
+            params += ("&mod=1&rid=" + getIcalUTC(item.recurrenceId)); // THIS INSTANCE
         }
         
         params += ("&method=" + method);
@@ -659,8 +660,7 @@ function calWcapCalendar_adoptItem(item, listener)
     
     try {
         if (!isParent(item)) {
-            this_.logError("adoptItem(): unexpected proxy!");
-            debugger;
+            logError("adoptItem(): unexpected proxy!", this);
         }
         this.storeItem(true/*bAddItem*/,
                        item, null, request,
@@ -781,7 +781,7 @@ function calWcapCalendar_deleteItem(item, listener)
         else { // delete THIS INSTANCE:
             var rid = item.recurrenceId;
             if (rid.isDate) {
-                // cs does not accept DATE:
+                // cs does not accept DATE here:
                 rid = rid.clone();
                 rid.isDate = false;
             }
@@ -843,17 +843,32 @@ calWcapCalendar.prototype.parseItems = function calWcapCalendar_parseItems(
                         log(attr + " is " + dt, this_);
                     }
                     var tzid = subComp.getFirstProperty(xprop);
-                    if (tzid != null) {
-                        subComp[attr] = dt.getInTimezone(tzid.value);
-                        if (LOG_LEVEL > 2) {
-                            log("patching " + xprop + " from " +
-                                dt + " to " + subComp[attr], this_);
+                    if (tzid) {
+                        var tz = this_.session.getTimezone(tzid.value);
+                        ASSERT(tz, "timezone not found: " + tzid);
+                        if (tz) {
+                            subComp[attr] = dt.getInTimezone(tz);
+                            if (LOG_LEVEL > 2) {
+                                log("patching " + xprop + ": from " +
+                                    dt + " to " + subComp[attr], this_);
+                            }
                         }
                     }
                 }
             }
 
             patchTimezone(subComp, "startTime", "X-NSCP-DTSTART-TZID");
+            patchTimezone(subComp, "recurrenceId", "X-NSCP-DTSTART-TZID");
+
+            var organizer = subComp.getFirstProperty("ORGANIZER");
+            if (organizer && organizer.getParameter("SENT-BY")) { // has SENT-BY
+                // &emailorcalid=1 sets wrong email, workaround setting calid...
+                var id = organizer.getParameter("X-S1CS-CALID");
+                if (id) {
+                    organizer.value = id;
+                }
+            }
+
             var item = null;
             switch (subComp.componentType) {
             case "VEVENT": {
@@ -903,7 +918,7 @@ calWcapCalendar.prototype.parseItems = function calWcapCalendar_parseItems(
                     item.recurrenceInfo = null;
                     if (LOG_LEVEL > 1) {
                         log("exception item: " + item.title +
-                            "\nrid=" + rid.icalString +
+                            "\nrid=" + getIcalUTC(rid) +
                             "\nitem.id=" + item.id, this_);
                     }
                     excItems.push(item);
@@ -943,7 +958,7 @@ calWcapCalendar.prototype.parseItems = function calWcapCalendar_parseItems(
         }
         else {
             logError("parseItems(): no parent item for " + item.title +
-                     ", rid=" + item.recurrenceId.icalString +
+                     ", rid=" + getIcalUTC(item.recurrenceId) +
                      ", item.id=" + item.id, this);
             // due to a server bug, in some scenarions the returned
             // data is lacking the parent item, leave parentItem open then
@@ -1017,7 +1032,8 @@ function calWcapCalendar_getItem(id, listener)
                 listener.onOperationComplete(
                     this_.superCalendar, getResultCode(err),
                     calIOperationListener.GET,
-                    item.id, err ? err : item);
+                    item ? item.id : null,
+                    err || item);
             }
             if (err)
                 this_.notifyError(err);
@@ -1050,8 +1066,10 @@ function calWcapCalendar_getItem(id, listener)
             request,
             function fetchEventById_resp(err, icalRootComp) {
                 if (err) {
-                    if (getResultCode(err) != calIWcapErrors.WCAP_FETCH_EVENTS_BY_ID_FAILED)
+                    if (!checkErrorCode(err, calIWcapErrors.WCAP_FETCH_EVENTS_BY_ID_FAILED) &&
+                        !checkErrorCode(err, calIWcapErrors.WCAP_COMPONENT_NOT_FOUND)) {
                         throw err;
+                    }
                     // try todos:
                     this_.issueNetworkRequest(
                         request,
@@ -1195,7 +1213,7 @@ function calWcapCalendar_getItems(itemFilter, maxResults, rangeStart, rangeEnd,
             request,
             function netResp(err, icalRootComp) {
                 if (err) {
-                    if (getResultCode(err) == calIWcapErrors.WCAP_ACCESS_DENIED_TO_CALENDAR) {
+                    if (checkErrorCode(err, calIWcapErrors.WCAP_ACCESS_DENIED_TO_CALENDAR)) {
                         // try free-busy times:
                         if (listener &&
                             (itemFilter & calICalendar.ITEM_FILTER_TYPE_EVENT) &&
@@ -1203,7 +1221,7 @@ function calWcapCalendar_getItems(itemFilter, maxResults, rangeStart, rangeEnd,
                         {
                             var freeBusyListener = { // calIGenericOperationListener:
                                 onResult: function freeBusyListener_onResult(request, result) {
-                                    if (!request.success)
+                                    if (!Components.isSuccessCode(request.status))
                                         throw request.status;
                                     var items = [];
                                     for each (var entry in result) {
@@ -1300,63 +1318,40 @@ function calWcapCalendar_getItems(itemFilter, maxResults, rangeStart, rangeEnd,
     return request;
 };
 
-// function calWcapSyncOperationListener() {
-//     this.superClass(respFunc);
-//     this.wrappedJSObject = this;
-// }
-// subClass(calWcapSyncOperationListener, calWcapRequest);
+calWcapCalendar.prototype.resetLog =
+function calWcapCalendar_resetLog()
+{
+    this.deleteProperty("replay.last_stamp");
+};
 
-// calWcapSyncOperationListener.prototype.QueryInterface =
-// function calWcapSyncOperationListener_QueryInterface(iid) {
-//     // xxx todo:
-//     const m_ifaces = [ Components.interfaces.nsISupports,
-//                        Components.interfaces.calIOperationListener,
-//                        Components.interfaces.calIWcapRequest ];
-//     qiface(m_ifaces, iid);
-//     return this;
-// };
-
-// // calIOperationListener:
-// calWcapSyncOperationListener.prototype.onOperationComplete =
-// function calWcapSyncOperationListener_onOperationComplete(
-//     calendar, status, opType, id, detail)
-// {
-//     if (status != NS_OK) {
-//         this.
-//             this.m_syncState.abort( detail );
-//     }
-//     else if (this.m_opType != opType) {
-//         this.m_syncState.abort(
-//             new Components.Exception("unexpected operation type: " +
-//                                      opType) );
-//     }
-//     this.m_syncState.release();
-// };
-
-// calWcapSyncOperationListener.prototype.onGetResult =
-// function calWcapSyncOperationListener_onGetResult(
-//     calendar, status, itemType, detail, count, items)
-// {
-//     this.m_syncState.abort(
-//         new Components.Exception("unexpected onGetResult()!") );
-// };
-
-// calWcapCalendar.prototype.syncChangesTo_resp = function(
-//     wcapResponse, syncState, listener, func )
-// {
-//     try {
-//         var icalRootComp = wcapResponse.data; // first statement, may throw
-//         var items = this.parseItems_(
-//             function(items) { items.forEach(func) },
-//             icalRootComp,
-//             calICalendar.ITEM_FILTER_ALL_ITEMS,
-//             0, null, null );
-//     }
-//     catch (exc) {
-//         syncState.abort( exc );
-//     }
-//     syncState.release();
-// };
+calWcapCalendar.prototype.replayChangesOn =
+function calWcapCalendar_replayChangesOn(destCal, listener)
+{
+    var this_ = this;
+    var opListener = {
+        QueryInterface: function(iid) {
+            return doQueryInterface(this, opListener, iid,
+                                    [Components.interfaces.calIGenericOperationListener,
+                                     Components.interfaces.nsISupports]);
+        },
+        onResult: function(op, result) {
+            if (!op.isPending) {
+                if (Components.isSuccessCode(op.status)) {
+                    this_.setProperty("replay.last_stamp", getIcalUTC(result));
+                    log("new replay stamp: " + getIcalUTC(result), this_);
+                } else {
+                    logError("error replaying changes: " + errorToString(op.status));
+                }
+                if (listener) {
+                    listener.onResult(op, null);
+                }
+            }
+        }
+    };
+    return this.syncChangesTo_(destCal, Components.interfaces.calICalendar.ITEM_FILTER_ALL_ITEMS,
+                               getDatetimeFromIcalString(this.getProperty("replay.last_stamp")),
+                               opListener);
+};
 
 calWcapCalendar.prototype.syncChangesTo =
 function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
@@ -1364,7 +1359,12 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
     // xxx todo: move to Thomas
     // do NOT puke up error box every three minutes!
     itemFilter |= calIWcapCalendar.ITEM_FILTER_SUPPRESS_ONERROR;
-    
+    return this.syncChangesTo_(destCal, itemFilter, dtFrom_, listener);
+};
+
+calWcapCalendar.prototype.syncChangesTo_ =
+function calWcapCalendar_syncChangesTo_(destCal, itemFilter, dtFrom_, listener)
+{
     var now = getTime(); // new stamp for this sync
     var this_ = this;
     var request_ = new calWcapRequest(
@@ -1372,18 +1372,32 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
             if (err) {
                 log("SYNC failed!", this_);
                 if (listener) {
-                    listener.onOperationComplete(
-                        this_.superCalendar, getResultCode(err),
-                        calIWcapCalendar.SYNC, null, err);
+                    var opListener = null;
+                    try {
+                        opListener = listener.QueryInterface(Components.interfaces.calIGenericOperationListener);
+                    } catch (exc) {
+                        listener.onOperationComplete(this_.superCalendar, getResultCode(err),
+                                                     calIWcapCalendar.SYNC, null, err);
+                    }
+                    if (opListener) {
+                        opListener.onResult(request, null);
+                    }
                 }
                 this_.notifyError(err, request.suppressOnError);
             }
             else {
                 log("SYNC succeeded.", this_);
                 if (listener) {
-                    listener.onOperationComplete(
-                        this_.superCalendar, NS_OK,
-                        calIWcapCalendar.SYNC, null, now);
+                    var opListener = null;
+                    try {
+                        opListener = listener.QueryInterface(Components.interfaces.calIGenericOperationListener);
+                    } catch (exc) {
+                        listener.onOperationComplete(this_.superCalendar, NS_OK,
+                                                     calIWcapCalendar.SYNC, null, now);
+                    }
+                    if (opListener) {
+                        opListener.onResult(request, now);
+                    }
                 }
             }
         },
@@ -1400,10 +1414,8 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
             // assure DATE-TIME:
             if (dtFrom.isDate)
                 dtFrom.isDate = false;
-            dtFrom = this.session.getServerTime(dtFrom);
         }
-        var zdtFrom = getIcalUTC(dtFrom);
-        
+
         var calObserver = null;
         if (listener) {
             try {
@@ -1412,21 +1424,26 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
             catch (exc) {
             }
         }
-        
+
+        var writeListener = {
+            onGetResult: function() {},
+            onOperationComplete: function(cal, status, opType, id, detail) {
+                if (!Components.isSuccessCode(status)) {
+                    request.execRespFunc(status); // any error on writing breaks whole operation
+                }
+            }
+        };
         var request = new calWcapRequest(
             function netFinishedRespFunc(err, data) {
                 var modifiedIds = {};
                 for each (var item in request.m_modifiedItems) {
                     var dtCreated = item.getProperty("CREATED");
-                    var bAdd = (!dtCreated || !dtFrom ||
-                                dtCreated.compare(dtFrom) >= 0);
+                    var bAdd = (!dtCreated || !dtFrom || dtCreated.compare(dtFrom) >= 0);
                     modifiedIds[item.id] = true;
                     if (bAdd) {
-                        // xxx todo: verify whether exceptions
-                        //           have been written
                         log("syncChangesTo(): new item " + item.id, this_);
                         if (destCal) {
-//                                 destCal.addItem(item, addItemListener);
+                            destCal.addItem(item, writeListener);
                         }
                         if (calObserver)
                             calObserver.onAddItem(item);
@@ -1434,7 +1451,7 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
                     else {
                         log("syncChangesTo(): modified item " + item.id, this_);
                         if (destCal) {
-//                             destCal.modifyItem(item, null, modifyItemListener);
+                            destCal.modifyItem(item, null, writeListener);
                         }
                         if (calObserver)
                             calObserver.onModifyItem(item, null);
@@ -1447,7 +1464,7 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
                     else if (isParent(item)) {
                         log("syncChangesTo(): deleted item " + item.id, this_);
                         if (destCal) {
-//                             destCal.deleteItem(item, deleteItemListener);
+                            destCal.deleteItem(item, writeListener);
                         }
                         if (calObserver)
                             calObserver.onDeleteItem(item);
@@ -1458,7 +1475,7 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
                         parent.recurrenceInfo.removeOccurrenceAt(item.recurrenceId);
                         log("syncChangesTo(): modified parent "+ parent.id, this_);
                         if (destCal) {
-//                             destCal.modifyItem(parent, item, deleteItemListener);
+                            destCal.modifyItem(parent, item, writeListener);
                         }
                         if (calObserver)
                             calObserver.onModifyItem(parent, item);
@@ -1466,38 +1483,56 @@ function calWcapCalendar_syncChangesTo(destCal, itemFilter, dtFrom_, listener)
                 }
             }, "syncChangesTo() netFinishedRespFunc");
         request_.attachSubRequest(request);
-        
-        var params = ("&relativealarm=1&compressed=1&recurring=1" +
-                      "&emailorcalid=1&fmt-out=text%2Fcalendar");
-        params += ("&dtstart=" + zdtFrom);
-        params += ("&dtend=" + getIcalUTC(this.session.getServerTime(now)));
-        
-        log("syncChangesTo(): getting last modifications...", this);
-        this.issueNetworkRequest(
+
+        // assure being logged in to calc server times:
+        this.session.getSessionId(
             request,
-            function modifiedNetResp(err, icalRootComp) {
-                if (err)
-                    throw err;
-                request.m_modifiedItems = this_.parseItems(
-                    icalRootComp, calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null);
-            },
-            stringToIcal, "fetchcomponents_by_lastmod",
-            params + getItemFilterParams(itemFilter),
-            calIWcapCalendar.AC_COMP_READ);        
-        
-        log("syncChangesTo(): getting deleted items...", this);
-        this.issueNetworkRequest(
-            request,
-            function modifiedNetResp(err, icalRootComp) {
-                if (err)
-                    throw err;
-                request.m_deletedItems = this_.parseItems(
-                    icalRootComp, calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null);
-            },
-            stringToIcal, "fetch_deletedcomponents",
-            params + getItemFilterParams(itemFilter & // only component types
-                                         calICalendar.ITEM_FILTER_TYPE_ALL),
-            calIWcapCalendar.AC_COMP_READ);
+            function getSessionId_resp(err, sessionId) {
+                try {
+                    if (err) {
+                        throw err;
+                    }
+                    var params = ("&relativealarm=1&compressed=1&recurring=1" +
+                                  "&emailorcalid=1&fmt-out=text%2Fcalendar");
+                    if (dtFrom) {
+                        dtFrom = this_.session.getServerTime(dtFrom);
+                    }
+                    params += ("&dtstart=" + getIcalUTC(dtFrom));
+                    params += ("&dtend=" + getIcalUTC(this_.session.getServerTime(now)));
+                    
+                    log("syncChangesTo(): getting last modifications...", this_);
+                    this_.issueNetworkRequest(
+                        request,
+                        function modifiedNetResp(err, icalRootComp) {
+                            if (err) {
+                                throw err;
+                            }
+                            request.m_modifiedItems = this_.parseItems(
+                                icalRootComp, calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null);
+                        },
+                        stringToIcal, "fetchcomponents_by_lastmod",
+                        params + getItemFilterParams(itemFilter),
+                        calIWcapCalendar.AC_COMP_READ);        
+
+                    log("syncChangesTo(): getting deleted items...", this_);
+                    this_.issueNetworkRequest(
+                        request,
+                        function modifiedNetResp(err, icalRootComp) {
+                            if (err) {
+                                throw err;
+                            }
+                            request.m_deletedItems = this_.parseItems(
+                                icalRootComp, calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null);
+                        },
+                        stringToIcal, "fetch_deletedcomponents",
+                        params + getItemFilterParams(itemFilter & // only component types
+                                                     calICalendar.ITEM_FILTER_TYPE_ALL),
+                        calIWcapCalendar.AC_COMP_READ);
+                }
+                catch (exc) {
+                    request.execRespFunc(exc);
+                }
+            });
     }
     catch (exc) {
         request_.execRespFunc(exc);
