@@ -45,17 +45,39 @@ const NS_ERROR_MODULE_BASE_OFFSET = 0x45;
 const NS_ERROR_MODULE_NETWORK = 6;
 
 function generateFailure(module, code) {
-    return ((1<<31) | ((module + NS_ERROR_MODULE_BASE_OFFSET) << 16) | code);
+    // 1<<31 generates negative number, so use literal, don't use logical operators:
+    return (0x80000000 + ((module + NS_ERROR_MODULE_BASE_OFFSET) << 16) + code);
 }
 
 function generateNetFailure(code) {
     return generateFailure(NS_ERROR_MODULE_NETWORK, code);
 }
 
-function getErrorModule(rc) {
+function getResultCode(err) {
+    if (err === undefined || err === null)
+        return NS_OK;
+    if (isNaN(err)) {
+        if (err instanceof nsIException)
+            return err.result;
+        else
+            return Components.results.NS_ERROR_FAILURE;
+    }
+    return err;
+}
+
+function getErrorModule(err) {
+    var rc = getResultCode(err);
     return (((rc >>> 16) & 0x7fff) - NS_ERROR_MODULE_BASE_OFFSET);
 }
 
+function checkErrorCode(err, rcBits, maskBits) {
+    if (!maskBits) {
+        maskBits = 0;
+    }
+    var rc = getResultCode(err);
+    return ((rc ^ rcBits) >>> maskBits) == 0;
+}
+
 // Cannot perform operation, because user is offline.
 // The following error codes have been adopted from
 // netwerk/base/public/nsNetError.h
@@ -336,19 +358,6 @@ function checkWcapIcalErrno(icalRootComp, expectedErrno) {
     checkWcapErrno(getWcapIcalErrno(icalRootComp), expectedErrno);
 }
 
-function getResultCode(err)
-{
-    if (err === undefined || err === null)
-        return NS_OK;
-    if (isNaN(err)) {
-        if (err instanceof nsIException)
-            return err.result;
-        else
-            return Components.results.NS_ERROR_FAILURE;
-    }
-    return err;
-}
-
 function errorToString(err)
 {
     if (err) {
